local frame = CreateFrame("FRAME")
frame:RegisterEvent("PLAYER_ENTERING_WORLD")
frame:RegisterEvent("ZONE_CHANGED")
frame:RegisterEvent("ZONE_CHANGED_NEW_AREA")
frame:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED")

local makeCampID = 312370
--local makeCampID = 212653 (For testing purposes)
local returnToCampID = 312372
local vulperaGreetingSoundID = 110866
local allowUnknownUpdate = false

local CHAT_COLOR_WHISPER = "|cffFF00EE"
local CHAT_COLOR_END = "|r"
local CAMPTWACKER_FEEDBACK_PREFIX = "[CampTwacker] A Friendly Vulpera whispers: "
local UNRELIABLE_CAMPSITE = "Your camp location was unknown when you cast Return to Camp. We "
    .. "tried to jot it down, but this method can be a little unreliable. Check the tooltip to make sure the new "
    .. "location is correct. If it isn't, please use Make Camp before you leave!"

-- Saves the camp location when Make Camp is used.
frame:SetScript("OnEvent", function(event, ...)
    local e = ...
    --print(e)
    if (e == "UNIT_SPELLCAST_SUCCEEDED") then
        local _, source, _, spellID = ...
        if ((source == "player") and (spellID == makeCampID)) then
            SaveCampsite()
        elseif ((source == "player") and (spellID == returnToCampID)) then
            if (campLocation == nil) then
                allowUnknownUpdate = true
            end
        end

    elseif ((e == "ZONE_CHANGED") or (e == "ZONE_CHANGED_NEW_AREA")) then
        if ((campLocation == nil) and (allowUnknownUpdate == true)) then
            SaveCampsite()
            PrepareChatMessage(UNRELIABLE_CAMPSITE)
        end

    elseif (e == "PLAYER_ENTERING_WORLD") then
        UnregisterEvents()
        PlaySound(vulperaGreetingSoundID)
    end
end)


function isVulpera()
    local _,currentRace = UnitRace("player")
    if (currentRace == "Vulpera") then
        return true
    else
        return false
    end
end


function SaveCampsite()
    local subzone = GetMinimapZoneText()
    local zone = GetZoneText()
    if (subzone == zone) then
        campLocation = zone
    else
        campLocation = subzone .. ", " .. zone
    end
    UnregisterEvents()
    allowUnknownUpdate = false
end



function PrepareChatMessage(msg)
    PlaySound(vulperaGreetingSoundID)
    print(CHAT_COLOR_WHISPER .. CAMPTWACKER_FEEDBACK_PREFIX .. msg .. CHAT_COLOR_END)
end

-- Unregisters events based on what is needed. All events are disabled if not vulpera.
function UnregisterEvents()
    if ((isVulpera()) and not (campLocation == nil)) then
        frame:UnregisterEvent("ZONE_CHANGED")
        frame:UnregisterEvent("ZONE_CHANGED_NEW_AREA")
    elseif (not isVulpera()) then
        frame:UnregisterEvent("PLAYER_ENTERING_WORLD")
        frame:UnregisterEvent("UNIT_SPELLCAST_SUCCEEDED")
        campLocation = nil  -- Deletes camp location in the event of race change in case they change back
    end
end

-- Appends camp location to Make Camp and Return to Camp spells.
local function ShowCampsite(tooltip, ...)
        local _,spellID = GameTooltip:GetSpell()
        if (spellID == makeCampID or spellID == returnToCampID) then
            if (campLocation == nil) then
                tooltip:AddLine("You seem to have forgotten\nwhere you last went camping...\n")
            else
                tooltip:AddLine("Your campsite is located in\n" .. campLocation .. ".\n")
            end
        end
end


if (isVulpera()) then  -- Just in case :)
    GameTooltip:HookScript("OnTooltipSetSpell", ShowCampsite)
end